#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const utils_1 = require("./utils");
const ora = require("ora");
exports.default = (modules, { dev, prod, yarn } = {}, shouldSpin = false) => __awaiter(this, void 0, void 0, function* () {
    // SPINNER
    const spinner = ora();
    const waitOn = (message) => {
        if (shouldSpin) {
            spinner.start(message);
        }
    };
    const succeed = () => {
        if (shouldSpin) {
            spinner.succeed();
        }
    };
    const fail = (e) => {
        if (shouldSpin) {
            spinner.fail();
        }
        console.error(e);
        process.exit(e.code);
    };
    // END OF SPINNER
    try {
        waitOn('Installing Packages');
        yield utils_1.npmInstall(modules, Boolean(dev), yarn);
    }
    catch (e) {
        fail(e);
    }
    succeed();
    const needsTypes = _.filter(yield Promise.all(modules.map(utils_1.missingTypes)));
    waitOn('Checking for @types');
    const typesToFetch = _.filter(yield Promise.all(needsTypes.map(m => utils_1.getTypingInfo(m))));
    succeed();
    try {
        waitOn('Installing Available Types');
        yield utils_1.npmInstall(typesToFetch.map(t => `@types/${t}`), !Boolean(prod), yarn);
    }
    catch (e) {
        fail(e);
    }
    succeed();
    const missing = _.difference(needsTypes, typesToFetch);
    utils_1.printPackages('\nThe following packages were fully installed', _.difference(modules, missing));
    utils_1.printPackages('The following packages were installed, but lack types', missing);
});
